@@ -65,15 +65,9 @@ config->{'device_auth'} = [ App::Netdisco::Util::SNMP::fixup_device_auth() ];
 
 # defaults for workers
 setting('workers')->{queue} ||= 'PostgreSQL';
-if (exists setting('workers')->{interactives}
-    or exists setting('workers')->{pollers}) {
-
-    setting('workers')->{tasks} =
-      (setting('workers')->{pollers} || 0)
-      + (setting('workers')->{interactives} || 0);
-
-    delete setting('workers')->{pollers};
-    delete setting('workers')->{interactives};
+if ($ENV{ND2_SINGLE_WORKER}) {
+  setting('workers')->{tasks} = 1;
+  delete config->{'schedule'};
 }
 
 # force skipped DNS resolution, if unset
@@ -133,6 +127,18 @@ config->{'schedule'} = config->{'housekeeping'}
   if setting('housekeeping') and not setting('schedule');
 delete config->{'housekeeping'};
 
+# used to have separate types of worker
+if (exists setting('workers')->{interactives}
+    or exists setting('workers')->{pollers}) {
+
+    setting('workers')->{tasks} ||=
+      (setting('workers')->{pollers} || 0)
+      + (setting('workers')->{interactives} || 0);
+
+    delete setting('workers')->{pollers};
+    delete setting('workers')->{interactives};
+}
+
 # schedule expire used to be called expiry
 setting('schedule')->{expire} ||= setting('schedule')->{expiry}
   if setting('schedule') and exists setting('schedule')->{expiry};
