@@ -32,10 +32,20 @@ register 'register_worker' => sub {
     my $job = shift or die 'missing job param';
     # use DDP; p $workerconf;
 
+    $ENV{ND2_LOG_PLUGINS} && debug sprintf '-> run worker %s/%s/%s',
+      @$workerconf{qw/phase namespace priority/};
+
     # update job's record of namespace and priority
     # check to see if this namespace has already passed at higher priority
     return if $job->namespace_passed($workerconf);
 
+    # support part-actions via action::namespace
+    if ($job->only_namespace and $workerconf->{phase} ne 'check') {
+      return unless $workerconf->{namespace} eq lc( $job->only_namespace )
+        or (($workerconf->{phase} eq 'early')
+             and ($job->device and not $job->device->in_storage));
+    }
+
     my @newuserconf = ();
     my @userconf = @{ setting('device_auth') || [] };
 
@@ -44,7 +54,7 @@ register 'register_worker' => sub {
       my $no   = (exists $workerconf->{no}   ? $workerconf->{no}   : undef);
       my $only = (exists $workerconf->{only} ? $workerconf->{only} : undef);
 
-      return $job->add_status( Status->noop('worker not applicable to this device') )
+      return $job->add_status( Status->info('worker not applicable to this device') )
         if ($no and check_acl_no($job->device, $no))
            or ($only and not check_acl_only($job->device, $only));
 
@@ -61,7 +71,7 @@ register 'register_worker' => sub {
       }
 
       # per-device action but no device creds available
-      return $job->add_status( Status->noop('worker driver or action not applicable') )
+      return $job->add_status( Status->info('worker driver or action not applicable') )
         if 0 == scalar @newuserconf;
     }
 
